
rm(list = ls())
library(tidyverse)
library(readxl)
library(pals)
library(paletteer)
library(robCompositions)
library(cluster)    # clustering algorithms
library(factoextra) # clustering algorithms & visualization
library(patchwork)
library(compositions)
library(ggdendro)
library(khroma)
library(marelac)
library(EnvStats)
library(patchwork)

dir <-
  paste0(dirname(rstudioapi::getSourceEditorContext()$path), "/")
setwd(dir)

# select elements used in cluster analysis, a compromise between maximizing the number of samples and maximizing the number of elements
ele <- c(
  "SiO2",
  "TiO2",
  "Al2O3",
  "FeOT",
  "MnO",
  "MgO",
  "CaO",
  "Na2O",
  "K2O",
  # "P2O5",
  # "Cs",
  "Rb",
  "Ba",
  "Sr",
  "La",
  "Ce",
  # "Pr",
  "Nd",
  # "Sm",
  "Eu",
  # "Gd",
  "Tb",
  # "Dy",
  "Ho",
  # "Er",
  # "Tm",
  "Yb",
  # "Lu",
  # "Y",
  "Zr",
  "Nb"#,
  # "Hf",
  # "Ta",
  # "Pb",
  # "Th",
  # "U"
  # "Sc",
  # "V",
  # "Cr",
  # "Co",
  # "Ni",
  # "Cu",
  # "Zn"
  # "Mo"
)

units <- c(rep("wt%",9),rep("ppm",length(ele)-9))
elements.select <- paste0(ele,"_",units)

# AVO database
avo <- read_excel("Database of volcanic geochemistry.xlsx",sheet="AVO", guess_max = 20000) %>%
  rename(id= AVO_ID) %>%
  mutate(source = "AVO")
# GEOROC database
ncvp <- read_excel("Database of volcanic geochemistry.xlsx",sheet="GEOROC", guess_max = 20000) %>%
  rename(id= GEOROC_ID) %>%
  mutate(source = "GEOROC")

# combine the data
all <- avo %>%
  mutate(label = str_trim(str_replace_all(
    paste(field, TAS, volcano, eruption, tephra), "NA", ""
  ))) %>%
  ungroup() %>%
  select(id, source,SampleID,label, field, TAS, element, unit,concentration,lat,lon,material) %>%
  mutate(id = as.character(id)) %>%
  full_join(
    ncvp %>%
      mutate(label = str_trim(str_replace_all(
        paste(field, TAS, location), "NA", ""
      ))) %>%
      ungroup() %>%
      select(id,source,SampleID, label, field, TAS, element,unit, concentration,lat,lon,material) %>%
      mutate(id = as.character(id))
  ) %>%
  mutate(element=paste0(element,"_",unit)) %>%
  select(-unit) %>%
  spread(key = element, value = concentration) %>%
  filter_at(vars(one_of(elements.select)), all_vars(!is.na(.)))

# Isometric log ratio transform of data before cluster analysis
dataMatrix <- all %>% ungroup() %>%
  mutate(rownames = paste(label, id, sep = "|")) %>%
  column_to_rownames("rownames") %>%
  select(one_of(elements.select)) %>%
  ilr() %>%
  as.matrix()


# Cluster analysis: complete linkage with euclidean distance
method = 'complete'
distance = "euclidean"

cluster.analysis <- hclust(dist(dataMatrix, distance), method)

# cut the cluster tree and decide the number of groups
k <- 15 # number of groups, 15 is big enough
cluster.group = cutree(cluster.analysis, k)


# summary of the clusters and rank them by frequency
cluster.group.rank = data.frame(table(cluster.group)) %>%
  mutate(rank = rank(-Freq, ties.method = "first")) %>%
  mutate(cluster.group = as.integer(cluster.group))


# add a cluster group column to the original data
# cluster groups are named as 1, 2, 3..., which are the ranks of the group
data.cluster <- all %>%
  add_column(cluster = cluster.group) %>%
  left_join(cluster.group.rank, by = c(cluster = "cluster.group")) %>%
  arrange(rank) %>%
  select(-cluster) %>%
  rename(cluster_rank=rank)


# dendroplot of the cluster analysis result
source("Dendroplot.R")

p.dendro <- plot.Dendro(cluster.analysis, data.cluster)

ggsave(
  p.dendro,
  device = "pdf",
  filename = paste("dendro", k, method, distance, ".pdf"),
  width = 50,
  height = 300,
  units = "cm",
  limitsize = FALSE
)


# some small clusters are ignored (if size <2% of the number of samples)
# name the clusters
cluster.select <- data.cluster %>%
  group_by(cluster_rank) %>%
  filter(Freq >= ceiling(dim(all)[1] * 0.02)) %>%
  mutate(
    cluster_label = case_when(
      cluster_rank == 1 ~ "Basalt-andesite-dacite",
      cluster_rank == 2 ~ "Basalt (tholeiitic)",
      cluster_rank == 3 ~ "Basalt (alkaline)",
      cluster_rank == 4 ~ "Rhyolite (adakite)",
      cluster_rank == 5 ~ "Rhyolite (typical arc)",
      TRUE ~ as.character(NA)
    )
  ) %>%
  write_csv("Clustered volcanic data.csv")

cluster.mean <- cluster.select %>%
  ungroup() %>%
  select(`Al2O3_wt%`:`Zr_ppm`, cluster_label) %>%
  gather(key = "element", value = "value", -cluster_label) %>%
  group_by(cluster_label, element) %>%
  summarize(conc = geoMean(value,na.rm = TRUE))  %>%
  spread(key = cluster_label, value = conc) %>%
  separate(element,c("element","unit"),sep="_") %>%
  arrange(element) %>%
  drop_na() %>%
  write_csv("Volcanic endmembers.csv")
