



rm(list = ls())
library(tidyverse)
library(writexl)
library(readxl)
library(pals)
library(limSolve)
library(patchwork)
library(robCompositions)

dir <-
  paste0(dirname(rstudioapi::getSourceEditorContext()$path), "/")
setwd(dir)


source("process GOA data.r")
source("process endmember data.r")

# geochemical data of samples
samples <-
  read_excel("Data for geochemical inversion.xlsx", sheet = "GOA.samples")

#sample information
sample.info <-
  read_excel("Data for geochemical inversion.xlsx", sheet = "GOA.samples.info") %>%
  rename(sample = name)

# units and relative standard deviations of measured concentrations for each element used in the inversion
elements <-
  read_excel("Data for geochemical inversion.xlsx", sheet = "elements")

# geochemical data of the volcanic and terrigenous endmembers
endmembers <-
  read_excel("Data for geochemical inversion.xlsx", sheet = "endmembers")


nend <- dim(endmembers)[2] - 2 # number of endmembers
nsample <- dim(samples)[2] - 2 # number of samples

sum(endmembers$element!=samples$element)
sum(endmembers$unit!=samples$unit)

sampleMatrix <- as.matrix(samples[,3:(nsample+2)])
endmemberMatrix <- as.matrix(endmembers[,3:(nend+2)])
# This function solves the non-negative weighted linear least square problem: min(||(Ax-B)/W||^2) and Gx>=H
# where x is a vector including the mixing fraction of the volcanic and terrigenous endmembers
# A is  geochemical compositions of the endmembers
# B is the measured geochemcal composition of the sediment samples from the Gulf of Alaska site 85JC/U1419
# W is the weighing vector equal to the standard deviations of elemental concentrations based on repeated digestion of sediment reference materials
# G is the identity matrix and H is zero. Gx>=H ensures that x is positive

fitlim <- function(data, parms,label) {
  with(as.list(parms), {

    W <- 1 / (c(rsd, 0.01) * c(data, 1))

    A = rbind(endmembers, rep(1, nend)) # matrix of the endmembers

    B = c(data, 1) # vector of data

    res <- lsei(
      A = A * W,
      B = B * W,
      G = diag(nend * 1),
      H = rep(0, nend * 1),
      type = 2,
      tol = 1e-12,
      fulloutput = TRUE,
      verbose = TRUE,
    )

    fraction <- data.frame(fraction = res$X) %>%
      rownames_to_column("endmember") %>%
      mutate(sample = label)

    residual <- data.frame(
      element = c(elements, "f_sum"),
      residual = (A %*% res$X - B),
      residual_weighted = (A %*% res$X - B) * W
    ) %>%
      mutate(sample = label)

    # error <- data.frame(var = c(colnames(endmembers)),
    #   error=sqrt(sum(((A %*% res$X - B) * W)^2)/(28-7)*diag(solve(t(A)%*%diag(W^2)%*%A))))

    return(list(fraction = fraction, residual = residual))
  })
}

parms = list(
  rsd = elements$rsd,
  endmembers = endmemberMatrix,
  nend = nend,
  elements = elements$element
)

fitlim(sampleMatrix[,1], parms,colnames(sampleMatrix)[1])


GOA.inversion <- lapply(
  1:nsample,
  FUN = function(i) {
    fitlim(sampleMatrix[, i], parms,colnames(sampleMatrix)[i])
  }
)

GOA.fraction <- lapply(GOA.inversion, function(x)
  x$fraction) %>%
  reduce(full_join) %>%
  left_join(sample.info) %>%
  mutate(group = ifelse(
    endmember %in% c("Terrigenous_HOL", "Terrigenous_LGM"),
    "Terrigenous",
    "Volcanic"
  )) %>%
  mutate(fraction = fraction*lith/100) %>%
  write_csv(paste0("GOA inversion fraction.csv"))


GOA.residual <- lapply(GOA.inversion, function(x)
  x$residual) %>%
  reduce(full_join) %>%
  left_join(sample.info) %>%
  mutate(element = factor(
    element,
    levels = c(
      "Al2O3",
      "FeOT",
      "MgO",
      "MnO",
      "Na2O",
      "K2O",
      "TiO2",
      "Cr",
      "Cu",
      "Ni",
      "V",
      "Zn",
      "Zr",
      "La",
      "Ce",
      "Pr",
      "Nd",
      "Sm",
      "Eu",
      "Gd",
      "Tb",
      "Dy",
      "Ho",
      "Er",
      "Tm",
      "Yb",
      "Lu",
      "f_sum"
    )
  )) %>%
  write_csv(paste0("GOA inversion residual.csv"))

