


rm(list = ls())
library(ncdf4)
library(tidyverse)
library(readxl)
library(writexl)
library(pals)
library(patchwork)
library(parallel)

dir <-
  paste0(dirname(rstudioapi::getSourceEditorContext()$path), "/")
setwd(dir)


lagged.timeseries <-
  read_excel("PISM sensitivity.xlsx", sheet = "lagged.timeseries") %>%
  # filter(model != "ODP1012") %>%
  mutate(model = paste0(model, ".", resolution)) %>%
  filter(age >= 0e3, age <= 26e3) %>%
  arrange(model,age)

SST.ensemble <-
  read_csv(paste0(dirname(dirname(dirname(
    dir
  ))), "/SST/BAYSPLINE-master/GOA.Ensemble.csv"))

SST <-
  read_csv(paste0(
    dirname(dirname(dirname(dir))),
    "/SST/BAYSPLINE-master/GOA.calibration.csv"
  )) %>%
  select(age) %>%
  add_column(SST.ensemble) %>%
  mutate(across(Var1:Var1000, function(x) {
    (x - mean(x[.$age <= 4e3])) * 1.12
    # mean(x[(.$age >= 22e3) & (.$age<=32e3)])
  })) %>%
  arrange(age) %>%
  filter(age<=26e3)

SST.mean <- SST %>%
  gather(-age, key = "rand", value = "SST.an") %>%
  group_by(age) %>%
  summarize(delT = mean(SST.an, na.rm = TRUE),delT.sd = sd(SST.an,na.rm = TRUE))

SST.mean.summ <- SST.mean %>%
  filter(age >= 22e3, age <= 32e3) %>%
  summarize(delT.g = mean(delT, na.rm = TRUE),delT.sd = sd(delT,na.rm = TRUE))

SST.mean.summ
#
ggplot(SST.mean) +
  geom_line(aes(age, delT))



predictIce <- function(model.i) {
  library(tidyr)
  library(dplyr)
  library(lokern)
  data <- lagged.timeseries %>%
    filter(model == model.i)

  lag <- data$lag.best.yr[1]

  reg <- lm(ice.volume.msle ~ delT.lag, data)
  rnormvec = function(j, se) {
    n = length(se)
    sapply(
      1:n,
      FUN = function(i) {
        rnorm(n = 1,
              mean = 0,
              sd = se[i])
      }
    )
  }

  delAge = 0.1e3
  age.out = seq(0e3, 24e3, delAge)

  predictvol <- function(i, j) {
    sstdata <- SST[, i + 1]
    colnames(sstdata) <- "delT.lag"
    res <-
      predict(reg, sstdata, interval = "confidence", se.fit = T)

    glkerns(
      SST$age - lag,
      res$fit[, 1] + rnormvec(j, res$se.fit),
      deriv = 0,
      x.out = age.out,
      hetero = TRUE,
      # bandwidth = 1.5e3,
      is.rand = FALSE,
      # m1 = 5,
      korder = 2,
    )$est


  }

  ice.vol = sapply(sample(1:1000, 100), function(i)
    sapply(1:100, function(j)
      predictvol(i, j)), simplify = "array")

  ice.vol2 = t(apply(ice.vol, 1, c))

  ice.rate.m = sapply(1:100 * 100, function(i) {
    -(lead(ice.vol2[, i], 1) - lag(ice.vol2[, i], 1)) / delAge/2*1e3
  })


  result <- data.frame(age = age.out, ice.vol2) %>%
    gather(key = "order", value = "ice.vol.n", -age) %>%
    left_join(
      data.frame(age = age.out, ice.rate.m) %>%
        gather(key = "order", value = "dvoldt.n", -age)
    ) %>%
    mutate(model = model.i)

  return(result)

}


models <- unique(lagged.timeseries$model)

cl <- makeCluster(8)
clusterExport(cl, c("lagged.timeseries", "SST", "models"))

pism = do.call(rbind.data.frame, parLapply(cl, models, predictIce))
stopCluster(cl)



pism.summary.model <- pism %>%
  group_by(age, model) %>%
  filter(age >= 0e3) %>%
  summarize(
    ice.vol = median(ice.vol.n) ,
    ice.vol.2.5 = quantile(ice.vol.n, 0.025),
    ice.vol.97.5 = quantile(ice.vol.n, 0.975) ,
    ice.vol.16 = quantile(ice.vol.n, 0.16),
    ice.vol.84 = quantile(ice.vol.n, 0.84) ,
    dvoldt = median(dvoldt.n, na.rm = TRUE),
    dvoldt.2.5 = quantile(dvoldt.n, 0.025, na.rm = TRUE) ,
    dvoldt.97.5 = quantile(dvoldt.n, 0.975, na.rm = TRUE),
    dvoldt.16 = quantile(dvoldt.n, 0.16, na.rm = TRUE) ,
    dvoldt.84 = quantile(dvoldt.n, 0.84, na.rm = TRUE),
  )

#
# ggplot(pism.summary.model, aes(age, ice.vol, color = model, fill = model)) +
#   geom_line() +
#   geom_ribbon(aes(ymin = ice.vol.2.5, ymax = ice.vol.97.5), alpha = 0.3) +
#   # geom_ribbon(aes(ymin = ice.vol.16, ymax = ice.vol.84), alpha = 0.6) +
#   scale_x_continuous(limits = c(0.5e3, 25e3)) +
#   scale_color_manual(values = okabe(8)) +
#   scale_fill_manual(values = okabe(8)) +
#   facet_grid(model ~ .)
#
#
# ggplot(pism.summary.model, aes(age, dvoldt, color = model, fill = model)) +
#   geom_line() +
#   geom_ribbon(aes(ymin = dvoldt.2.5, ymax = dvoldt.97.5), alpha = 0.3) +
#   # geom_ribbon(aes(ymin = ice.vol.16, ymax = ice.vol.84), alpha = 0.6) +
#   scale_x_continuous(limits = c(0.5e3, 25e3)) +
#   scale_color_manual(values = okabe(8)) +
#   scale_fill_manual(values = okabe(8)) +
#   facet_grid(model ~ .)


pism.summary <- pism %>%
  ungroup() %>%
  filter(model!="ODP1012.10km") %>%
  group_by(age) %>%
  filter(age >= 0e3) %>%
  summarize(
    ice.vol = median(ice.vol.n) ,
    ice.vol.2.5 = quantile(ice.vol.n, 0.025),
    ice.vol.97.5 = quantile(ice.vol.n, 0.975) ,
    ice.vol.16 = quantile(ice.vol.n, 0.16),
    ice.vol.84 = quantile(ice.vol.n, 0.84) ,
    dvoldt = median(dvoldt.n, na.rm = TRUE),
    dvoldt.2.5 = quantile(dvoldt.n, 0.025, na.rm = TRUE) ,
    dvoldt.97.5 = quantile(dvoldt.n, 0.975, na.rm = TRUE),
    dvoldt.16 = quantile(dvoldt.n, 0.16, na.rm = TRUE) ,
    dvoldt.84 = quantile(dvoldt.n, 0.84, na.rm = TRUE),
  )
#
# ggplot(pism.summary, aes(age, ice.vol)) +
#   geom_line() +
#   geom_ribbon(aes(ymin = ice.vol.2.5, ymax = ice.vol.97.5), alpha = 0.3) +
#   geom_ribbon(aes(ymin = ice.vol.16, ymax = ice.vol.84), alpha = 0.6) +
#   scale_x_continuous(limits = c(0.5e3, 25e3))


ggplot(pism.summary, aes(age, dvoldt)) +
  geom_line() +
  # geom_ribbon(aes(ymin = dvoldt.2.5, ymax = dvoldt.97.5), alpha = 0.3) +
  geom_ribbon(aes(ymin = dvoldt.16, ymax = dvoldt.84), alpha = 0.6) +
  scale_x_continuous(limits = c(0, 25e3)) +
  scale_y_continuous(limits= c(-4,2))






write_csv(pism.summary, "PISM ice vol GOA SST.CSV")

write_csv(pism.summary.model, "PISM ice vol GOA SST model.CSV")

